

/* Copyright (c) {{ year }} {{ author }}. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

This is a UI file (.ui.qml) that is intended to be edited in Qt Design Studio only.
It is supposed to be strictly declarative and only uses a subset of QML. If you edit
this file manually, you might introduce QML code that is not supported by Qt Design Studio.
Check out https://doc.qt.io/qtcreator/creator-quick-ui-forms.html for details on .ui.qml files.
*/
import QtQuick 2.15

Grid {
    id: mainMenu

    columns: 3
    rows: 0
    spacing: 20
    visible: opacity !== 0

    // export the buttons, so that we can attach
    // behaviours to them from the parent component
    property alias btn_1: btn_1
    property alias btn_2: btn_2
    property alias btn_3: btn_3
    property alias btn_4: btn_4
    property alias btn_5: btn_5
    property alias btn_6: btn_6

    IconButton {
        id: btn_1
        icon: "images/push.svg"
        label: "Change title"
    }

    IconButton {
        id: btn_2
        icon: "images/push.svg"
        label: "Display image"
    }

    IconButton {
        id: btn_3
        icon: "images/push.svg"
        label: "Show alert"
    }

    IconButton {
        id: btn_4
        icon: "images/push.svg"
        label: "Publish ROS value"
    }

    IconButton {
        id: btn_5
        icon: "images/push.svg"
        label: "Start task 1"
    }

    IconButton {
        id: btn_6
        icon: "images/push.svg"
        label: "Start unknown task"
    }
}
