<!DOCTYPE html>
<html>
<head>
	<title></title>
	<!-- css -->
	<link rel="stylesheet" href="style/Montserrat.css">
	<link rel="stylesheet" href="style/style.css"> 
	<!-- js -->
	<script src="js/lib/jquery.js"></script>
	<script type="module" src="js/main.js"></script>
	<meta name="viewport" content="width=device-width, user-scalable=no">
    <meta http-equiv="content-type" content="text/html; charset=UTF-8" /> 
</head>
<body>
	<div class="main-container">
		<div class="question-text" id="question"></div>
		<div class="buttons-grid" id="answers_list">
			<!--<div class="button-item" id="answer_1">Option 1</div>-->
		</div>
	</div>
	<div class="exit-button" id="exit_btn">Exit</div>
</body>
</html>


