# Spanish (Spain) translations for {{ id }}.
# Copyright (C) 2024 ORGANIZATION
# This file is distributed under the same license as the {{ id }} project.
# FIRST AUTHOR <EMAIL@ADDRESS>, 2024.
#
msgid ""
msgstr ""
"Project-Id-Version: {{ id }} 0.1.0\n"
"Report-Msgid-Bugs-To: EMAIL@ADDRESS\n"
"POT-Creation-Date: 2024-11-21 17:05+0100\n"
"PO-Revision-Date: 2024-11-22 09:20+0100\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
"Language: es_ES\n"
"Language-Team: es_ES <LL@li.org>\n"
"Plural-Forms: nplurals=2; plural=(n != 1)\n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=utf-8\n"
"Content-Transfer-Encoding: 8bit\n"
"Generated-By: Babel 2.8.0\n"

msgid "Hello, what can I do for you today?"
msgstr "Hola, cómo te puedo ayudar hoy?"

