# Sample skill API.
#
# Define your skill's API here, eg pass custom data to configure your skill, or
# return specific values: feel free to modify it as needed.
#
# Do not forget to update the skill manifest (defined in package.xml) to
# reflect the changes made here and document them.
#
# The `meta`, `result` and `feedback` fields are reserved for the standard
# and must always be present.

std_skills/Meta meta
string skill_data
---
std_skills/Result result
string value
---
std_skills/Feedback feedback

