.pragma library
var width = 1024;
var height = 768;
var relativeFontDirectory = "fonts"

var font = Qt.font({
    family: "Inter",
    pixelSize: 20,
});
var largeFont = Qt.font({
    family: "Inter",
    pixelSize: 50,
    bold: true
});

var bgColor= "#122930"; // PAL midnight blue
var fgColor = "#f4f4f4"; // PAL cool gray
var accentColor = "#ffa77c"; // PAL peach
